<?php
class ControllerPaymentKredivo extends Controller
{

    private $error = array();

    public function index()
    {
        $this->language->load('payment/kredivo');

        $this->document->setTitle($this->language->get('heading_title'));

        $this->load->model('setting/setting');
        $this->load->model('localisation/order_status');
        $this->config->get('curency');

        if (($this->request->server['REQUEST_METHOD'] == 'POST') && $this->validate()) {
            $this->model_setting_setting->editSetting('kredivo', $this->request->post);

            $this->session->data['success'] = $this->language->get('text_success');

            $this->response->redirect($this->url->link('extension/payment', 'token=' . $this->session->data['token'], 'SSL'));
        }

        $language_entries = array(

            'heading_title',
            'text_enabled',
            'text_disabled',
            'text_yes',
            'text_live',
            'text_successful',
            'text_fail',
            'text_all_zones',
            'text_edit',

            'entry_api_version',
            'entry_environment',
            'entry_server_key',
            'entry_test',
            'entry_total',
            'entry_order_status',
            'entry_geo_zone',
            'entry_status',
            'entry_sort_order',
            'entry_currency_conversion',
            'entry_success_mapping',
            'entry_failure_mapping',
            'entry_pending_mapping',
            'entry_challenge_mapping',
            'entry_display_name',

            'button_save',
            'button_cancel',
        );

        foreach ($language_entries as $language_entry) {
            $data[$language_entry] = $this->language->get($language_entry);
        }

        if (isset($this->error)) {
            $data['error'] = $this->error;
        } else {
            $data['error'] = array();
        }

        $data['breadcrumbs'] = array();

        $data['breadcrumbs'][] = array(
            'text'      => $this->language->get('text_home'),
            'href'      => $this->url->link('common/home', 'token=' . $this->session->data['token'], 'SSL'),
            'separator' => false,
        );

        $data['breadcrumbs'][] = array(
            'text'      => $this->language->get('text_payment'),
            'href'      => $this->url->link('extension/payment', 'token=' . $this->session->data['token'], 'SSL'),
            'separator' => ' :: ',
        );

        $data['breadcrumbs'][] = array(
            'text'      => $this->language->get('heading_title'),
            'href'      => $this->url->link('payment/kredivo', 'token=' . $this->session->data['token'], 'SSL'),
            'separator' => ' :: ',
        );

        $data['action'] = $this->url->link('payment/kredivo', 'token=' . $this->session->data['token'], 'SSL');
        $data['cancel'] = $this->url->link('extension/payment', 'token=' . $this->session->data['token'], 'SSL');

        $inputs = array(
            'kredivo_api_version',
            'kredivo_environment',
            'kredivo_server_key',
            'kredivo_test',
            'kredivo_order_status_id',
            'kredivo_geo_zone_id',
            'kredivo_sort_order',
            'kredivo_currency_conversion',
            'kredivo_status',
            'kredivo_success_mapping',
            'kredivo_failure_mapping',
            'kredivo_pending_mapping',
            'kredivo_challenge_mapping',
            'kredivo_display_name',
            'kredivo_enabled_payments',
            'kredivo_sanitization',
        );

        foreach ($inputs as $input) {
            if (isset($this->request->post[$input])) {
                $data[$input] = $this->request->post[$input];
            } else {
                $data[$input] = $this->config->get($input);
            }
        }

        $this->load->model('localisation/order_status');
        $data['order_statuses'] = $this->model_localisation_order_status->getOrderStatuses();

        $this->load->model('localisation/geo_zone');
        $data['geo_zones'] = $this->model_localisation_geo_zone->getGeoZones();

        $this->template      = 'payment/kredivo.tpl';
        $data['column_left'] = $this->load->controller('common/column_left');
        $data['header']      = $this->load->controller('common/header');
        $data['footer']      = $this->load->controller('common/footer');

        if (!$this->currency->has('IDR')) {
            $data['curr'] = true;
        } else {
            $data['curr'] = false;
        }

        $this->response->setOutput($this->load->view('payment/kredivo.tpl',$data));
    }

    protected function validate()
    {

        if (!$this->user->hasPermission('modify', 'payment/kredivo')) {
            $this->error['warning'] = $this->language->get('error_permission');
        }

        if (!$this->request->post['kredivo_display_name']) {
            $this->error['display_name'] = $this->language->get('error_display_name');
        }

        if (!$this->request->post['kredivo_environment']) {
            $this->request->post['kredivo_environment'] = 1;
        }

        if (!$this->request->post['kredivo_server_key']) {
            $this->error['server_key'] = $this->language->get('error_server_key');
        }

        if (!$this->request->post['kredivo_currency_conversion'] && !$this->currency->has('IDR')) {
            $this->error['currency_conversion'] = $this->language->get('error_currency_conversion');
        }

        if (!$this->error) {
            return true;
        } else {
            return false;
        }
    }
}
