<?php

require_once DIR_SYSTEM . 'library/kredivo/autoload.php';

class ControllerPaymentKredivo extends Controller
{

    public function index()
    {

        $data['errors'] = array();

        $data['button_confirm'] = $this->language->get('button_confirm');
        $data['pay_type']       = $this->config->get('kredivo_payment_type');
        $data['text_loading']   = $this->language->get('text_loading');
        $data['process_order']  = $this->url->link('payment/kredivo/process_order');

        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/kredivo.tpl')) {
            return $this->load->view($this->config->get('config_template') . '/template/payment/kredivo.tpl', $data);
        } else {
            if (VERSION > 2.1) {
                return $this->load->view('payment/kredivo', $data);
            } else {
                return $this->load->view('default/template/payment/kredivo.tpl', $data);
            }
        }
    }

    /**
     * Called when a customer checkouts.
     * If it runs successfully, it will redirect to VT-Web payment page.
     */
    public function process_order()
    {
        // $this->load->model('payment/kredivo');
        $this->load->model('checkout/order');
        $this->load->language('payment/kredivo');

        $data['errors']         = array();
        $data['button_confirm'] = $this->language->get('button_confirm');

        $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
        $this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('kredivo_pending_mapping'));

        $transaction_details                 = array();
        $transaction_details['order_id']     = $this->session->data['order_id'];
        $transaction_details['gross_amount'] = $order_info['total'];

        // ====================== TRANSACTION ======================
        $items = $this->_calculateProducts($this->cart);

        // convert all item prices to IDR
        if ($this->config->get('config_currency') != 'IDR') {
            if ($this->currency->has('IDR')) {
                foreach ($items as &$item) {
                    $item['price'] = intval($this->currency->convert($item['price'], $this->config->get('config_currency'), 'IDR'));
                }
                unset($item);

                $transaction_details['gross_amount'] = intval($this->currency->convert($transaction_details['gross_amount'], $this->config->get('config_currency'), 'IDR'));
            } else if ($this->config->get('kredivo_currency_conversion') > 0) {
                foreach ($items as &$item) {
                    $item['price'] = intval($item['price'] * $this->config->get('kredivo_currency_conversion'));
                }
                unset($item);

                $transaction_details['gross_amount'] = intval($transaction_details['gross_amount'] * $this->config->get('kredivo_currency_conversion'));
            } else {
                $data['errors'][] = "Either the IDR currency is not installed or "
                    . "the Kredivo currency conversion rate is valid. "
                    . "Please review your currency setting.";
            }
        }

        $total_price = 0;
        foreach ($items as $item) {
            $total_price += $item['price'] * $item['quantity'];
        }

        if ($total_price != $transaction_details['gross_amount']) {
            $items[] = array(
                'id'       => 'discount',
                'name'     => 'Coupon',
                'price'    => $transaction_details['gross_amount'] - $total_price,
                'quantity' => 1,
            );
        }

        $params_transaction_details = array(
            'order_id' => (string) $transaction_details['order_id'],
            'amount'   => $total_price,
            'items'    => $items,
        );

        // ====================== CUSTOMER ======================
        $params_customer = array(
            'first_name' => $order_info['payment_firstname'],
            'last_name'  => $order_info['payment_lastname'],
            'email'      => $order_info['email'],
            'phone'      => $order_info['telephone'],
        );

        // ====================== BILLING ADDRESS ======================
        $params_billing_address = array(
            'first_name'   => $order_info['payment_firstname'],
            'last_name'    => $order_info['payment_lastname'],
            'address'      => $order_info['payment_address_1'],
            'city'         => $order_info['payment_city'],
            'postal_code'  => $order_info['payment_postcode'],
            'phone'        => $order_info['telephone'],
            'country_code' => strlen($order_info['payment_iso_code_3'] != 3) ? 'IDN' : $order_info['payment_iso_code_3'],
        );

        // ====================== SHIPPING ADDRESS ======================
        $params_shipping_address = array();
        if ($this->cart->hasShipping()) {
            $params_shipping_address = array(
                'first_name'   => $order_info['shipping_firstname'],
                'last_name'    => $order_info['shipping_lastname'],
                'address'      => $order_info['shipping_address_1'],
                'city'         => $order_info['shipping_city'],
                'postal_code'  => $order_info['shipping_postcode'],
                'phone'        => $order_info['telephone'],
                'country_code' => strlen($order_info['payment_iso_code_3'] != 3) ? 'IDN' : $order_info['payment_iso_code_3'],
            );
        } else {
            $params_shipping_address = $params_billing_address;
        }

        try {

            // Setup Config
            Kredivo_Config::$is_production = $this->_isProduction();

            $params = array(
                'server_key'          => $this->config->get('kredivo_server_key'),
                'payment_type'        => '30_days',
                'transaction_details' => $params_transaction_details,
                'customer_details'    => $params_customer,
                'billing_address'     => $params_billing_address,
                'shipping_address'    => $params_shipping_address,
                'push_uri'            => $this->_pushNotificationUri(),
                'order_status_uri'    => $this->_orderStatusUri(),
                'back_to_store_uri'   => $this->_backToStoreUri(),
            );

            // echo json_encode($params);
            // echo "<pre>", print_r($params);
            // exit();

            Kredivo_Log::debug(array_merge([
                'production' => Kredivo_Config::$is_production,
                'endpoint'   => Kredivo_Config::get_api_endpoint(),
            ], $params));

            $this->response->setOutput(Kredivo_Api::checkout_url($params));

        } catch (Exception $e) {}

    }

    public function landing()
    {
        $redirUrl = $this->config->get('config_ssl');
        error_log('redir url: ' . $redirUrl); //debugan

        if (isset($_GET['order_id']) && isset($_GET['tr_status']) && in_array($_GET['tr_status'], array('capture', 'pending', 'settlement'))) {
            $this->cart->clear();
            $redirUrl = $this->url->link('checkout/success&');
            $this->response->redirect($redirUrl);

        } else if (isset($_GET['order_id']) && isset($_GET['tr_status']) && $_GET['tr_status'] == 'deny') {
            $redirUrl = $this->url->link('checkout/cart');
            $this->response->redirect($redirUrl);

        } else if (isset($_GET['order_id']) && !isset($_GET['tr_status'])) {
            $redirUrl = $this->url->link('checkout/cart');
            $this->response->redirect($redirUrl);
        }

        $this->response->redirect($redirUrl);
    }

    public function status()
    {
        $this->load->model('checkout/order');

        if ($_SERVER['REQUEST_METHOD'] == 'POST') {

            $input = new Kredivo_Input();
            error_log("Kredivo : " . $input->get_json());
            Kredivo_Log::debug($input->get_json());

            $order    = $this->model_checkout_order->getOrder($input->order_id);
            $response = array(
                'message' => 'Detail order status',
                'order'   => array(
                    'status'    => strtoupper($order['order_status']),
                    'timestamp' => $order['date_modified'],
                ),
            );

            echo Kredivo_Api::response($response);
            exit();

        }
    }

    public function notification()
    {
        if ($_SERVER['REQUEST_METHOD'] == 'POST') {

            Kredivo_Api::close_connection(Kredivo_Api::response());

            $input = new Kredivo_Input();
            error_log("Kredivo : " . $input->get_json());
            Kredivo_Log::debug($input->get_json());

            // Setup Config
            Kredivo_Config::$is_production = $this->_isProduction();

            try {
                $confirmation = Kredivo_Api::confirm_order(array(
                    'transaction_id' => $input->transaction_id,
                    'signature_key'  => $input->signature_key,
                ));

                error_log("Kredivo : " . json_encode($confirmation));
                Kredivo_Log::debug($confirmation);

                if (strtolower($confirmation->status) == 'ok') {
                    $this->_validConfirmation($confirmation);
                }
            } catch (Exception $e) {
                Kredivo_Log::debug($e->getMessage());
            }

            exit();
        }
    }

    private function _validConfirmation($confirmation)
    {
        $this->load->model('checkout/order');

        $fraud_status = strtolower($confirmation->fraud_status);
        if ($fraud_status == 'accept') {
            $transaction_status = strtolower($confirmation->transaction_status);
            switch ($transaction_status) {
                case 'settlement':
                    $this->model_checkout_order->addOrderHistory(
                        $confirmation->order_id,
                        $this->config->get('kredivo_success_mapping'),
                        'Kredivo payment successful.');
                    break;
                case 'cancel':
                    $this->model_checkout_order->addOrderHistory(
                        $confirmation->order_id,
                        $this->config->get('kredivo_failure_mapping'),
                        'Kredivo payment canceled.');
                    break;
                case 'pending':
                    $this->model_checkout_order->addOrderHistory(
                        $confirmation->order_id,
                        $this->config->get('kredivo_pending_mapping'),
                        'Kredivo payment pending.');
                    break;
                default:
                    $this->model_checkout_order->addOrderHistory(
                        $confirmation->order_id,
                        $this->config->get('kredivo_pending_mapping'),
                        'Kredivo payment pending.');
                    break;
            }
        }

    }

    private function _calculateProducts($cart)
    {
        $this->load->model('catalog/product');
        $this->load->model('catalog/category');

        $items    = array();
        $products = $cart->getProducts();

        if (sizeof($products) > 0) {
            foreach ($products as $item) {

                if ($item['quantity']) {

                    $categories = $this->model_catalog_product->getCategories($item['product_id']);
                    if ($categories) {
                        $categories_info = $this->model_catalog_category->getCategory($categories[0]['category_id']);
                    }

                    if (($this->config->get('config_customer_price') && $this->customer->isLogged()) || !$this->config->get('config_customer_price')) {
                        $item['price'] = $this->tax->calculate($item['price'], $item['tax_class_id'], $this->config->get('config_tax'));
                    }

                    $items[] = array(
                        "id"       => $item['product_id'],
                        "name"     => $item['name'],
                        "price"    => $this->_determineValidNumber($item['price']),
                        "quantity" => $this->_determineValidNumber($item['quantity']),
                        "type"     => isset($categories_info['name']) ? $categories_info['name'] : '',
						"url"      => $this->url->link('product/product', 'product_id=' .  $item['product_id']),
                    );
                }
            }
        }

        if ($cart->hasShipping()) {
            $shipping_info = $this->session->data['shipping_method'];
            if (($this->config->get('config_customer_price') && $this->customer->isLogged()) || !$this->config->get('config_customer_price')) {
                $shipping_info['cost'] = $this->tax->calculate($shipping_info['cost'], $shipping_info['tax_class_id'], $this->config->get('config_tax'));
            }

            $items[] = array(
                'id'       => 'shippingfee',
                'name'     => 'Shipping Fee',
                'price'    => $this->_determineValidNumber($shipping_info['cost']),
                'quantity' => 1,
            );
        }

        return $items;
    }

    private function _determineValidNumber($str)
    {
        try {
            return is_string($str) ? floatval($str) : $str;
        } catch (Exception $e) {}

        return $str;
    }

    private function _isProduction()
    {
        return ($this->config->get('kredivo_environment') == 'production') ? true : false;
    }

    private function _backToStoreUri()
    {
        return $this->url->link('payment/kredivo/landing');
    }

    private function _orderStatusUri()
    {
        return $this->url->link('payment/kredivo/status');
    }

    private function _pushNotificationUri()
    {
        return $this->url->link('payment/kredivo/notification');
    }
}
